@@ -1,4 +1,4 @@
-import logging
+import logging, copy
 
 from casbin.effect import Effector, get_effector, effect_to_bool
 from casbin.model import Model, FunctionMap
@@ -8,6 +8,18 @@ from casbin.rbac import default_role_manager
 from casbin.util import generate_g_function, SimpleEval, util
 
 
+class EnforceContext:
+    """
+    EnforceContext is used as the first element of the parameter "rvals" in method "enforce"
+    """
+
+    def __init__(self, rtype: str, ptype: str, etype: str, mtype: str):
+        self.rtype: str = rtype
+        self.ptype: str = ptype
+        self.etype: str = etype
+        self.mtype: str = mtype
+
+
 class CoreEnforcer:
     """CoreEnforcer defines the core functionality of an enforcer."""
 
@@ -53,7 +65,11 @@ class CoreEnforcer:
     def init_with_model_and_adapter(self, m, adapter=None):
         """initializes an enforcer with a model and a database adapter."""
 
-        if not isinstance(m, Model) or adapter is not None and not isinstance(adapter, Adapter):
+        if (
+            not isinstance(m, Model)
+            or adapter is not None
+            and not isinstance(adapter, Adapter)
+        ):
             raise RuntimeError("Invalid parameters for enforcer.")
 
         self.adapter = adapter
@@ -71,7 +87,7 @@ class CoreEnforcer:
 
     def _initialize(self):
         self.rm_map = dict()
-        self.eft = get_effector(self.model.model["e"]["e"].value)
+        self.eft = get_effector(self.model["e"]["e"].value)
         self.watcher = None
 
         self.enabled = True
@@ -131,37 +147,70 @@ class CoreEnforcer:
 
     def get_role_manager(self):
         """gets the current role manager."""
-        return self.rm_map['g']
+        return self.rm_map["g"]
+
+    def get_named_role_manager(self, ptype):
+        if ptype in self.rm_map.keys():
+            return self.rm_map.get(ptype)
+        raise ValueError("ptype not found")
 
     def set_role_manager(self, rm):
         """sets the current role manager."""
-        self.rm_map['g'] = rm
+        self.rm_map["g"] = rm
+
+    def set_named_role_manager(self, ptype, rm):
+        self.rm_map[ptype] = rm
 
     def set_effector(self, eft):
         """sets the current effector."""
-
         self.eft = eft
 
     def clear_policy(self):
-        """ clears all policy."""
+        """clears all policy."""
 
         self.model.clear_policy()
 
     def init_rm_map(self):
-        if 'g' in self.model.model.keys():
-            for ptype in self.model.model['g']:
-                self.rm_map[ptype] = default_role_manager.RoleManager(10)
+        if "g" in self.model.keys():
+            for ptype in self.model["g"]:
+                assertion = self.model["g"][ptype]
+                if assertion.value.count("_") == 2:
+                    self.rm_map[ptype] = default_role_manager.RoleManager(10)
+                else:
+                    self.rm_map[ptype] = default_role_manager.DomainManager(10)
 
     def load_policy(self):
         """reloads the policy from file/database."""
+        need_to_rebuild = False
+        new_model = copy.copy(self.model)
+        new_model.clear_policy()
 
-        self.model.clear_policy()
-        self.adapter.load_policy(self.model)
+        try:
 
-        self.init_rm_map()
-        self.model.print_policy()
-        if self.auto_build_role_links:
-            self.build_role_links()
+            self.adapter.load_policy(new_model)
+
+            new_model.sort_policies_by_priority()
+
+            self.init_rm_map()
+            self.model.print_policy()
+
+            if self.auto_build_role_links:
+
+                need_to_rebuild = True
+                for rm in self.rm_map.values():
+                    rm.clear()
+
+                new_model.build_role_links(self.rm_map)
+                self.build_role_links()
+
+            self.model = new_model
+
+        except Exception as e:
+
+            if self.auto_build_role_links and need_to_rebuild:
+                self.build_role_links()
+
+            raise e
 
     def load_filtered_policy(self, filter):
         """reloads a filtered policy from file/database."""
@@ -171,6 +220,9 @@ class CoreEnforcer:
             raise ValueError("filtered policies are not supported by this adapter")
 
         self.adapter.load_filtered_policy(self.model, filter)
+
+        self.model.sort_policies_by_priority()
+
         self.init_rm_map()
         self.model.print_policy()
         if self.auto_build_role_links:
@@ -233,11 +285,20 @@ class CoreEnforcer:
 
     def add_named_domain_matching_func(self, ptype, fn):
         """add_named_domain_matching_func add MatchingFunc by ptype to RoleManager"""
-        try:
+        if ptype in self.rm_map.keys():
             self.rm_map[ptype].add_domain_matching_func(fn)
             return True
-        except:
-            return False
+
+        return False
+
+    def new_enforce_context(self, suffix: str) -> EnforceContext:
+
+        return EnforceContext(
+            rtype="r" + suffix,
+            ptype="p" + suffix,
+            etype="e" + suffix,
+            mtype="m" + suffix,
+        )
 
     def enforce(self, *rvals):
         """decides whether a "subject" can access a "object" with the operation "action",
@@ -252,29 +313,43 @@ class CoreEnforcer:
         return judge result with reason
         """
 
+        rtype = "r"
+        ptype = "p"
+        etype = "e"
+        mtype = "m"
+
         if not self.enabled:
             return [False, []]
 
         functions = self.fm.get_functions()
 
-        if "g" in self.model.model.keys():
-            for key, ast in self.model.model["g"].items():
+        if "g" in self.model.keys():
+            for key, ast in self.model["g"].items():
                 rm = ast.rm
                 functions[key] = generate_g_function(rm)
 
-        if "m" not in self.model.model.keys():
+        if len(rvals) != 0:
+            if isinstance(rvals[0], EnforceContext):
+                enforce_context = rvals[0]
+                rtype = enforce_context.rtype
+                ptype = enforce_context.ptype
+                etype = enforce_context.etype
+                mtype = enforce_context.mtype
+                rvals = rvals[1:]
+
+        if "m" not in self.model.keys():
             raise RuntimeError("model is undefined")
 
-        if "m" not in self.model.model["m"].keys():
+        if "m" not in self.model["m"].keys():
             raise RuntimeError("model is undefined")
 
-        r_tokens = self.model.model["r"]["r"].tokens
-        p_tokens = self.model.model["p"]["p"].tokens
+        r_tokens = self.model["r"][rtype].tokens
+        p_tokens = self.model["p"][ptype].tokens
 
         if len(r_tokens) != len(rvals):
             raise RuntimeError("invalid request size")
 
-        exp_string = self.model.model["m"]["m"].value
+        exp_string = self.model["m"][mtype].value
         has_eval = util.has_eval(exp_string)
         if not has_eval:
             expression = self._get_expression(exp_string, functions)
@@ -283,11 +358,11 @@ class CoreEnforcer:
 
         r_parameters = dict(zip(r_tokens, rvals))
 
-        policy_len = len(self.model.model["p"]["p"].policy)
+        policy_len = len(self.model["p"][ptype].policy)
 
         explain_index = -1
         if not 0 == policy_len:
-            for i, pvals in enumerate(self.model.model["p"]["p"].policy):
+            for i, pvals in enumerate(self.model["p"][ptype].policy):
                 if len(p_tokens) != len(pvals):
                     raise RuntimeError("invalid policy size")
 
@@ -296,7 +371,10 @@ class CoreEnforcer:
 
                 if util.has_eval(exp_string):
                     rule_names = util.get_eval_value(exp_string)
-                    rules = [util.escape_assertion(p_parameters[rule_name]) for rule_name in rule_names]
+                    rules = [
+                        util.escape_assertion(p_parameters[rule_name])
+                        for rule_name in rule_names
+                    ]
                     exp_with_rule = util.replace_eval(exp_string, rules)
                     expression = self._get_expression(exp_with_rule, functions)
 
@@ -313,8 +391,9 @@ class CoreEnforcer:
                 else:
                     raise RuntimeError("matcher result should be bool, int or float")
 
-                if "p_eft" in parameters.keys():
-                    eft = parameters["p_eft"]
+                p_eft_key = ptype + "_eft"
+                if p_eft_key in parameters.keys():
+                    eft = parameters[p_eft_key]
                     if "allow" == eft:
                         policy_effects.add(Effector.ALLOW)
                     elif "deny" == eft:
@@ -324,17 +403,22 @@ class CoreEnforcer:
                 else:
                     policy_effects.add(Effector.ALLOW)
 
-                if self.eft.intermediate_effect(policy_effects) != Effector.INDETERMINATE:
+                if (
+                    self.eft.intermediate_effect(policy_effects)
+                    != Effector.INDETERMINATE
+                ):
                     explain_index = i
                     break
 
         else:
             if has_eval:
-                raise RuntimeError("please make sure rule exists in policy when using eval() in matcher")
+                raise RuntimeError(
+                    "please make sure rule exists in policy when using eval() in matcher"
+                )
 
             parameters = r_parameters.copy()
 
-            for token in self.model.model["p"]["p"].tokens:
+            for token in self.model["p"][ptype].tokens:
                 parameters[token] = ""
 
             result = expression.eval(parameters)
@@ -361,7 +445,7 @@ class CoreEnforcer:
 
         explain_rule = []
         if explain_index != -1 and explain_index < policy_len:
-            explain_rule = self.model.model["p"]["p"].policy[explain_index]
+            explain_rule = self.model["p"][ptype].policy[explain_index]
 
         return result, explain_rule
 
