@@ -1,9 +1,10 @@
 from casbin.management_enforcer import ManagementEnforcer
-from casbin.util import join_slice, set_subtract
+from casbin.util import join_slice, array_remove_duplicates, set_subtract
+
 
 class Enforcer(ManagementEnforcer):
     """
-        Enforcer = ManagementEnforcer + RBAC_API + RBAC_WITH_DOMAIN_API
+    Enforcer = ManagementEnforcer + RBAC_API + RBAC_WITH_DOMAIN_API
     """
 
     """creates an enforcer via file or DB.
@@ -16,15 +17,15 @@ class Enforcer(ManagementEnforcer):
     """
 
     def get_roles_for_user(self, name):
-        """ gets the roles that a user has. """
+        """gets the roles that a user has."""
         return self.model.model["g"]["g"].rm.get_roles(name)
 
     def get_users_for_role(self, name):
-        """ gets the users that has a role. """
+        """gets the users that has a role."""
         return self.model.model["g"]["g"].rm.get_users(name)
 
     def has_role_for_user(self, name, role):
-        """ determines whether a user has a role. """
+        """determines whether a user has a role."""
         roles = self.get_roles_for_user(name)
         return any(r == role for r in roles)
 
@@ -109,7 +110,7 @@ class Enforcer(ManagementEnforcer):
         """
         return self.has_policy(join_slice(user, *permission))
 
-    def get_implicit_roles_for_user(self, name, domain=None):
+    def get_implicit_roles_for_user(self, name, domain=""):
         """
         gets implicit roles that a user has.
         Compared to get_roles_for_user(), this function retrieves indirect roles besides direct roles.
@@ -135,7 +136,9 @@ class Enforcer(ManagementEnforcer):
 
         return res
 
-    def get_implicit_permissions_for_user(self, user, domain=None):
+    def get_implicit_permissions_for_user(
+        self, user, domain="", filter_policy_dom=True
+    ):
         """
         gets implicit permissions for a user or role.
         Compared to get_permissions_for_user(), this function retrieves permissions for inherited roles.
@@ -146,6 +149,9 @@ class Enforcer(ManagementEnforcer):
 
         get_permissions_for_user("alice") can only get: [["alice", "data2", "read"]].
         But get_implicit_permissions_for_user("alice") will get: [["admin", "data1", "read"], ["alice", "data2", "read"]].
+
+        Inherited roles can be matched by domain.
+        filter_policy_dom: bool - For given *domain*, policies will be filtered by domain as well. Default = True
         """
         roles = self.get_implicit_roles_for_user(user, domain)
 
@@ -153,7 +159,7 @@ class Enforcer(ManagementEnforcer):
 
         res = []
         for role in roles:
-            if domain:
+            if domain and filter_policy_dom:
                 permissions = self.get_permissions_for_user_in_domain(role, domain)
             else:
                 permissions = self.get_permissions_for_user(role)
@@ -173,13 +179,15 @@ class Enforcer(ManagementEnforcer):
         get_implicit_users_for_permission("data1", "read") will get: ["alice", "bob"].
         Note: only users will be returned, roles (2nd arg in "g") will be excluded.
         """
-        subjects = self.get_all_subjects()
-        roles = self.get_all_roles()
-
-        users = set_subtract(subjects, roles)
+        p_subjects = self.get_all_subjects()
+        g_inherit = self.model.get_values_for_field_in_policy("g", "g", 1)
+        g_subjects = self.model.get_values_for_field_in_policy("g", "g", 0)
+        subjects = array_remove_duplicates(g_subjects + p_subjects)
 
         res = list()
-        for user in users:
+        subjects = set_subtract(subjects, g_inherit)
+
+        for user in subjects:
             req = join_slice(user, *permission)
             allowed = self.enforce(*req)
 
@@ -190,11 +198,11 @@ class Enforcer(ManagementEnforcer):
 
     def get_roles_for_user_in_domain(self, name, domain):
         """gets the roles that a user has inside a domain."""
-        return self.model.model['g']['g'].rm.get_roles(name, domain)
+        return self.model.model["g"]["g"].rm.get_roles(name, domain)
 
     def get_users_for_role_in_domain(self, name, domain):
         """gets the users that has a role inside a domain."""
-        return self.model.model['g']['g'].rm.get_users(name, domain)
+        return self.model.model["g"]["g"].rm.get_users(name, domain)
 
     def add_role_for_user_in_domain(self, user, role, domain):
         """adds a role for a user inside a domain."""
