@@ -2,14 +2,15 @@ from . import Assertion
 from casbin import util, config
 from .policy import Policy
 
+
 class Model(Policy):
 
     section_name_map = {
-        'r': 'request_definition',
-        'p': 'policy_definition',
-        'g': 'role_definition',
-        'e': 'policy_effect',
-        'm': 'matchers',
+        "r": "request_definition",
+        "p": "policy_definition",
+        "g": "role_definition",
+        "e": "policy_effect",
+        "m": "matchers",
     }
 
     def _load_assertion(self, cfg, sec, key):
@@ -27,15 +28,15 @@ class Model(Policy):
 
         if "r" == sec or "p" == sec:
             ast.tokens = ast.value.split(",")
-            for i,token in enumerate(ast.tokens):
+            for i, token in enumerate(ast.tokens):
                 ast.tokens[i] = key + "_" + token.strip()
         else:
             ast.value = util.remove_comments(util.escape_assertion(ast.value))
 
-        if sec not in self.model.keys():
-            self.model[sec] = {}
+        if sec not in self.keys():
+            self[sec] = {}
 
-        self.model[sec][key] = ast
+        self[sec][key] = ast
 
         return True
 
@@ -75,6 +76,25 @@ class Model(Policy):
 
     def print_model(self):
         self.logger.info("Model:")
-        for k, v in self.model.items():
+        for k, v in self.items():
             for i, j in v.items():
                 self.logger.info("%s.%s: %s", k, i, j.value)
+
+    def sort_policies_by_priority(self):
+        for ptype, assertion in self["p"].items():
+            for index, token in enumerate(assertion.tokens):
+                if token == f"{ptype}_priority":
+                    assertion.priority_index = index
+                    break
+
+            if assertion.priority_index == -1:
+                continue
+
+            assertion.policy = sorted(
+                assertion.policy, key=lambda x: x[assertion.priority_index]
+            )
+
+            for i, policy in enumerate(assertion.policy):
+                assertion.policy_map[",".join(policy)] = i
+
+        return None
